<?php
namespace Drupal\time_diff\TwigExtension;

use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;
use DateTime;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;

class TimeDiffTwigExtension extends AbstractExtension {
  use StringTranslationTrait;

  protected $translation;

  // Inject the translation service.
  public function __construct(TranslationInterface $translation) {
    $this->translation = $translation;
  }

  /**
   * {@inheritdoc}
   */
  public function getFilters() {
    return [
      new TwigFilter('time_diff', [$this, 'calculateTimeDiff']),
    ];
  }

  /**
   * Calculate the human-readable time difference between two DateTime objects or timestamps.
   *
   * @param \DateTime|string|int $start
   *   The start date.
   * @param \DateTime|string|int|null $end
   *   The end date. Defaults to the current date and time if not provided.
   *
   * @return string
   *   The formatted time difference.
   *
   * @throws \Exception
   */
  public function calculateTimeDiff($start, $end = null) {
    if (!$start instanceof DateTime) {
      $start = new DateTime(is_numeric($start) ? "@$start" : $start);
    }

    if (!$end) {
      $end = new DateTime();
    } elseif (!$end instanceof DateTime) {
      $end = new DateTime(is_numeric($end) ? "@$end" : $end);
    }

    $interval = $start->diff($end);

    if ($interval->y > 0) {
      return $this->translation->formatPlural($interval->y, '1 year ago', '@count years ago');
    } elseif ($interval->m > 0) {
      return $this->translation->formatPlural($interval->m, '1 month ago', '@count months ago');
    } elseif ($interval->d > 0) {
      return $this->translation->formatPlural($interval->d, '1 day ago', '@count days ago');
    } elseif ($interval->h > 0) {
      return $this->translation->formatPlural($interval->h, '1 hour ago', '@count hours ago');
    } elseif ($interval->i > 0) {
      return $this->translation->formatPlural($interval->i, '1 minute ago', '@count minutes ago');
    } else {
      return $this->translation->translate('just now');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'time_diff.twig_extension';
  }
}